#!/usr/bin/env python
#
#       ladpsa_to_pd.py
#       
#       Copyright 2008 David Konsumer <konsumer@jetboystudio.com>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

"""
Create a series of puredata patches from ladpsa plugins

requires http://python-ladspa.sourceforge.net/

"""

import ladspa

pl=ladspa.PluginLibrary()

types={
  9:'ain',
  10:'aout',
  5:'cin',
  6:'cout',
}


class Pd:
  def __init__(self, descriptor):
    self.descriptor = descriptor
    self.name = descriptor.Attrs[1]
    self.label = descriptor.Attrs[3]
    self.io = descriptor.Attrs[6]
    self.guts = """
#N canvas 381 276 546 300 10;
#X obj -288 214 plugin~ %s;
#N canvas 3 26 450 300 controls 0;
#X obj -3 227 outlet;
""" % self.name
    self.controls=''
    
    print self.name
    print '-------------'
  
  def write(self):
    f=open('ladspa/%s.pd' % self.name,'wb')
    f.write(self.guts + "\n#X restore -288 189 pd controls;\n" + self.controls + "\n#X connect 1 0 0 0;\n#X coords 0 -1 1 1 137 %d 1 100 100;\n" % (12+(self.cin_count*20)) )
    f.close()
  
  def process(self):
    cin_count=0
    cin_c_count=1
    c_count=1
    ain_count=0
    aout_count=0
    cout_count=0
    object_count=0
    
    for interface in self.io:      
      object_count+=1
      i_name = interface[0]
      i_type = types[interface[1]]
      i_default = interface[2]
      i_min = interface[3]
      i_max = interface[4]
      
      print "%s [%s] (%f-%f) : %f" % (i_name, i_type, i_min, i_max, i_default)
      
      if (i_type == 'cin'):
        # if both zero, just set it to a "big" range
        if (i_min == 0 and i_min == 0):
          i_min = -999
          i_max = 999
        cin_count+=1
        self.controls+= "#X obj 105 %d hsl 128 15 0 127 0 0 \$0/control/%d \$1/control/%d %s 0 8 0 10 -260097 -1 -1 0 1;\n" %((100+(cin_count*17)), cin_count, cin_count, i_name.replace(' ','_'))
        self.guts+= "#X obj %d 0 r \$0/control/%d;\n" % ((cin_count*130), cin_count)
        self.guts+= "#X obj %d 20 / 127;\n" % (cin_count*130)
        self.guts+= "#X obj %d 40 notescale %f %f;\n" % ((cin_count*130), i_min, i_max)
        self.guts+= "#X msg %d 60 control #%d \$1;\n" % ((cin_count*130), cin_count)
        
        self.guts+= "#X connect %d 0 %d 0;\n" % (cin_c_count, cin_c_count+1)
        self.guts+= "#X connect %d 0 %d 0;\n" % (cin_c_count+1, cin_c_count+2)
        self.guts+= "#X connect %d 0 %d 0;\n" % (cin_c_count+2, cin_c_count+3)
        self.guts+= "#X connect %d 0 0 0;\n" % (cin_c_count+3)        
        
        cin_c_count+=4
        c_count+=1
      
      elif (i_type == 'ain'):
        ain_count+=1
        c_count+=1
        self.controls+='#X obj %d 132 inlet~ %s;\n' % ((ain_count*100) - 350, i_name)
        self.controls+="#X connect %d 0 0 %d;\n" % (c_count, ain_count)
      
      elif (i_type == 'aout'):
        aout_count+=1
        c_count+=1
        self.controls+='#X obj %d 300 outlet~ %s;\n' % ((aout_count*100) - 350, i_name)
        self.controls+="#X connect 0 %d %d 0;\n" % (aout_count, c_count)
      
      """
      doesn't seem to work right...
      elif (i_type == 'cout'):
        cout_count+=1
        c_count+=1
        self.controls+='#X obj %d 250 outlet %s;\n' % ((cout_count*100) - 350, i_name)
        self.controls+="#X connect 0 1 %d 0;\n" % (c_count)
      """
    
    self.cin_count = cin_count

for plugin in pl.Plugins:
  for descriptor in plugin.Descriptors:
    p = Pd(descriptor)
    p.process()
    print ""
    p.write()
